#ifndef __CHAR_H__
#define __CHAR_H__

#include <ctype.h>

#define CHAR_ATTR_NORMAL_F		0x00000001
#define CHAR_ATTR_BOLD_F		0x00000002
#define CHAR_ATTR_UNDERLINE_F	0x00000004
#define CHAR_ATTR_BLINK_F		0x00000008
#define CHAR_ATTR_REVERSE_F		0x00000010
#define CHAR_ATTR_STANDOUT_F	0x00000020
#define CHAR_ATTR_TRANSPARENT_F	0x00000040

#define CHAR_ESC_NON_F			0x00000000
#define CHAR_ESC_ESC_F			0x00000001
#define CHAR_ESC_ALT_F			0x00000002
#define CHAR_ESC_SHIFT_F		0x00000004
#define CHAR_ESC_CONTROL_F		0x00000008
#define CHAR_ESC_UP_F			0x00000010
#define CHAR_ESC_DOWN_F			0x00000020
#define CHAR_ESC_LEFT_F			0x00000040
#define CHAR_ESC_RIGHT_F		0x00000080
#define CHAR_ESC_F1_F			0x00000100
#define CHAR_ESC_F2_F			0x00000200
#define CHAR_ESC_F3_F			0x00000400
#define CHAR_ESC_F4_F			0x00000800
#define CHAR_ESC_F5_F			0x00001000
#define CHAR_ESC_F6_F			0x00002000
#define CHAR_ESC_F7_F			0x00004000
#define CHAR_ESC_F8_F			0x00008000
#define CHAR_ESC_F9_F			0x00010000
#define CHAR_ESC_F10_F			0x00020000
#define CHAR_ESC_F11_F			0x00040000
#define CHAR_ESC_F12_F			0x00080000
#define CHAR_ESC_DEL_F			0x00100000
#define CHAR_ESC_INS_F			0x00200000
#define CHAR_ESC_HOME_F			0x00400000
#define CHAR_ESC_END_F			0x00800000
#define CHAR_ESC_PGUP_F			0x01000000
#define CHAR_ESC_PGDN_F			0x02000000
#define CHAR_ESC_BKSP_F			0x04000000
#define CHAR_ESC_ENTER_F		0x08000000
#define CHAR_ESC_PRINTABLE_F	0x10000000
#define CHAR_ESC_UNK_F			0x80000000

#define CHAR_MODIFIER			((CHAR_ESC_ALT_F) || (CHAR_ESC_SHIFT_F) || (CHAR_ESC_CONTROL))

enum CharColor {cc_undef=0, cc_black,cc_red,cc_green,cc_yellow,
				cc_blue,cc_magenta,cc_cyan,cc_white, cc_last=32};

class Char {
	public:
		Char(int chr=0x20):
				_fg(cc_undef),_bg(cc_undef),_chr(chr),_attr(CHAR_ATTR_NORMAL_F),
				_ctrl(CHAR_ESC_NON_F|CHAR_ESC_PRINTABLE_F) { validateChr(chr); }
		Char(const Char &chr):
				_fg(chr._fg),_bg(chr._bg),_chr(chr._chr),_attr(chr._attr),
				_ctrl(chr._ctrl) { }
		~Char() { }
		Char &operator=(const Char &chr) {
				_fg=chr._fg; _bg=chr._bg; _chr=chr._chr; _attr=chr._attr;
				_ctrl=chr._ctrl;
				return (*this);
		}
		Char &operator=(int chr) { validateChr(chr); return (*this); }

		bool operator==(const Char &chr) const {
				if ((_fg==chr._fg) && (_bg==chr._bg) && (_attr==chr._attr) &&
					(_chr==chr._chr) && (_ctrl==chr._ctrl)) return (true);
				else return (false);
		}
		bool operator!=(const Char &chr) const { return !(operator==(chr)); }
		bool operator==(int chr) const { return (_chr==chr); }
		bool operator!=(int chr) const { return !(operator==(chr)); }
		bool actsLike(const Char &chr) const  {
				if ((_chr==chr._chr) && (_ctrl==chr._ctrl)) return (true);
				else return (false);
		}
		bool isLike(const Char &chr) const  {
				if ((_fg==chr._fg) && (_bg==chr._bg) && (_attr==chr._attr))
				return (true); else return (false);
		}
		bool setLike(const Char &chr) {
				_fg=chr._fg; _bg=chr._bg; _attr=chr._attr; return (true); 
		}

		bool tNormal() {
				return ((_attr^=CHAR_ATTR_NORMAL_F)&CHAR_ATTR_NORMAL_F);
		}
		bool isNormal() const { return (_attr&CHAR_ATTR_NORMAL_F); }
		bool tBold() { return ((_attr^=CHAR_ATTR_BOLD_F)&CHAR_ATTR_BOLD_F); }
		bool isBold() const { return (_attr&CHAR_ATTR_BOLD_F); }
		bool tUnderline(){
				return((_attr^=CHAR_ATTR_UNDERLINE_F)&CHAR_ATTR_UNDERLINE_F); }
		bool isUnderline() const { return(_attr&CHAR_ATTR_UNDERLINE_F); }
		bool tBlink() { return ((_attr^=CHAR_ATTR_BLINK_F)&CHAR_ATTR_BLINK_F); }
		bool isBlink() const { return (_attr&CHAR_ATTR_BLINK_F); }
		bool tReverse() {
				return ((_attr^=CHAR_ATTR_REVERSE_F)&CHAR_ATTR_REVERSE_F); }
		bool isReverse() const { return (_attr&CHAR_ATTR_REVERSE_F); }
		bool tStandout() {
				return ((_attr^=CHAR_ATTR_STANDOUT_F)&CHAR_ATTR_STANDOUT_F); }
		bool isStandout() const { return (_attr&CHAR_ATTR_STANDOUT_F); }
		CharColor setFore(CharColor fg) { return ((_fg=fg)); }
		CharColor getFore() const { return (_fg); }
		CharColor setBack(CharColor bg) { return ((_bg=bg)); }
		CharColor getBack() const { return (_bg); }
		void swap() { CharColor t=_fg; _fg=_bg; _bg=t; }
		int getSymbol() const { return (_chr); }
		int getUpperSymbol() const { 
			if ((_chr > 0x60) && (_chr < 0x7B)) return (_chr-0x20);
			else return (_chr);
		}
		bool isAlpha() const { return ((isPrintable())&&(isalpha(_chr))); }
		bool isAlNum() const { return ((isPrintable())&&(isalnum(_chr))); }
		bool isWhitespace() const { return ((isBlank())||(_chr=='\n')); }
		bool isBlank() const { return 
				((_chr==0x20) || (_chr==0x09)); }
		void setTransparent() { _attr=CHAR_ATTR_TRANSPARENT_F; }
		void unsetTransparent() { _attr=CHAR_ATTR_NORMAL_F; }
		bool isTransparent() const { return (_attr&CHAR_ATTR_TRANSPARENT_F); }

		bool isPrintable() const {  return (_ctrl&CHAR_ESC_PRINTABLE_F); }
		bool setUnknown() const { return (_ctrl^CHAR_ESC_UNK_F); }
		bool isExtended() const { return
				(_ctrl&(CHAR_ESC_ALT_F|CHAR_ESC_SHIFT_F|CHAR_ESC_CONTROL_F)); }
		void setAlt() { _ctrl|=CHAR_ESC_ALT_F; setNonPrintable(); }
		bool isAlt() { return (_ctrl&CHAR_ESC_ALT_F); }
		void setShift() { _ctrl|=CHAR_ESC_SHIFT_F; }
		bool isShift() { return (_ctrl&CHAR_ESC_SHIFT_F); }
		void setControl() { _ctrl|=CHAR_ESC_CONTROL_F; setNonPrintable(); }
		void unsetControl() { _ctrl^=CHAR_ESC_CONTROL_F; setNonPrintable(); }
		bool isControl() const { return (_ctrl&CHAR_ESC_CONTROL_F); }
		void setEsc() { _ctrl|=CHAR_ESC_ESC_F; setNonPrintable(); }
		bool isEsc() const { return (_ctrl&CHAR_ESC_ESC_F); }
		void setInterrupt() { _chr='c'; setControl(); }
		bool isInterrupt() { return ((_ctrl&CHAR_ESC_CONTROL_F)&&(_chr=='c')); }
		void setUpArrow() { _ctrl|=CHAR_ESC_UP_F; setNonPrintable(); }
		bool isUpArrow() const { return (_ctrl&CHAR_ESC_UP_F); }
		void setDownArrow() { _ctrl|=CHAR_ESC_DOWN_F; setNonPrintable(); }
		bool isDownArrow() const { return (_ctrl&CHAR_ESC_DOWN_F); }
		void setRightArrow() { _ctrl|=CHAR_ESC_RIGHT_F; setNonPrintable(); }
		bool isRightArrow() const { return (_ctrl&CHAR_ESC_RIGHT_F); }
		void setLeftArrow() { _ctrl|=CHAR_ESC_LEFT_F; setNonPrintable(); }
		bool isLeftArrow() const { return (_ctrl&CHAR_ESC_LEFT_F); }
		void setF1() { _ctrl|=CHAR_ESC_F1_F; setNonPrintable(); }
		bool isF1() const { return (_ctrl&CHAR_ESC_F1_F); }
		void setF2() { _ctrl|=CHAR_ESC_F2_F; setNonPrintable(); }
		bool isF2() const { return (_ctrl&CHAR_ESC_F2_F); }
		void setF3() { _ctrl|=CHAR_ESC_F3_F; setNonPrintable(); }
		bool isF3() const { return (_ctrl&CHAR_ESC_F3_F); }
		void setF4() { _ctrl|=CHAR_ESC_F4_F; setNonPrintable(); }
		bool isF4() const { return (_ctrl&CHAR_ESC_F4_F); }
		void setF5() { _ctrl|=CHAR_ESC_F5_F; setNonPrintable(); }
		bool isF5() const { return (_ctrl&CHAR_ESC_F5_F); }
		void setF6() { _ctrl|=CHAR_ESC_F6_F; setNonPrintable(); }
		bool isF6() const { return (_ctrl&CHAR_ESC_F6_F); }
		void setF7() { _ctrl|=CHAR_ESC_F7_F; setNonPrintable(); }
		bool isF7() const { return (_ctrl&CHAR_ESC_F7_F); }
		void setF8() { _ctrl|=CHAR_ESC_F8_F; setNonPrintable(); }
		bool isF8() const { return (_ctrl&CHAR_ESC_F8_F); }
		void setF9() { _ctrl|=CHAR_ESC_F9_F; setNonPrintable(); }
		bool isF9() const { return (_ctrl&CHAR_ESC_F9_F); }
		void setF10() { _ctrl|=CHAR_ESC_F10_F; setNonPrintable(); }
		bool isF10() const { return (_ctrl&CHAR_ESC_F10_F); }
		void setF11() { _ctrl|=CHAR_ESC_F11_F; setNonPrintable(); }
		bool isF11() const { return (_ctrl&CHAR_ESC_F11_F); }
		void setF12() { _ctrl|=CHAR_ESC_F12_F; setNonPrintable(); }
		bool isF12() const { return (_ctrl&CHAR_ESC_F12_F); }
		void setDel() { _ctrl|=CHAR_ESC_DEL_F; setNonPrintable(); }
		bool isDel() const { return (_ctrl&CHAR_ESC_DEL_F); }
		void setIns() { _ctrl|=CHAR_ESC_INS_F; setNonPrintable(); }
		bool isIns() const { return (_ctrl&CHAR_ESC_INS_F); }
		void setHome() { _ctrl|=CHAR_ESC_HOME_F; setNonPrintable(); }
		bool isHome() const { return (_ctrl&CHAR_ESC_HOME_F); }
		void setEnd() { _ctrl|=CHAR_ESC_END_F; setNonPrintable(); }
		bool isEnd() const { return (_ctrl&CHAR_ESC_END_F); }
		void setPgUp() { _ctrl|=CHAR_ESC_PGUP_F; setNonPrintable(); }
		bool isPgUp() const { return (_ctrl&CHAR_ESC_PGUP_F); }
		void setPgDn() { _ctrl|=CHAR_ESC_PGDN_F; setNonPrintable(); }
		bool isPgDn() const { return (_ctrl&CHAR_ESC_PGDN_F); }
		void setBksp() { _ctrl|=CHAR_ESC_BKSP_F; setNonPrintable(); }
		bool isBksp() const { return (_ctrl&CHAR_ESC_BKSP_F); }
		void setEnter() { _ctrl|=CHAR_ESC_ENTER_F; setNonPrintable(); }
		bool isEnter() const { return (_ctrl&CHAR_ESC_ENTER_F); }
	private:
		CharColor _fg;
		CharColor _bg;
		int _chr;
		unsigned int _attr;
		unsigned int _ctrl;
		void validateChr(int chr) {
			if ((chr < 0x20) || ((chr > 0x7E) && (chr < 0xA1)) || (chr > 0xFE))
				setNonPrintable();
			else 
				setPrintable();
			_chr=chr;
		}
		void setPrintable() {
			if (!(_ctrl&CHAR_ESC_PRINTABLE_F))
				_ctrl^=CHAR_ESC_PRINTABLE_F;
		}
		void setNonPrintable() {
			if (_ctrl&CHAR_ESC_PRINTABLE_F)
				_ctrl^=CHAR_ESC_PRINTABLE_F;
		}
};

#endif //__CHAR_H__
